# http://www.toukei.metro.tokyo.jp/jsuikei/js-index.htm
class PopulationItem
  def initialize(row)
    @name = row.getCell(1).getStringCellValue
    @total = row.getCell(2).getNumericCellValue.to_i
    @men = row.getCell(3).getNumericCellValue.to_i
    @women = row.getCell(4).getNumericCellValue.to_i
  end

  attr_reader :name, :total, :men, :women
end

class DistrictPopulation
  include Enumerable

  def initialize(j)
    @jinsui = j
  end
  def each
    @jinsui.each do |r|
      if r.getCell(1) && r.getRowNum < 100 &&
          /.+区$/ =~ r.getCell(1).getStringCellValue
        yield PopulationItem.new(r)
      end
    end
  end
end

class Jinsui
  include GetText::Rails 
  bindtextdomain 'population'

  include Enumerable
  include Rjb

  def initialize(name)
    raise name unless File.exists?(name)
    is = FileInputStream.new(name)
    begin
      fs = POIFSFileSystem.new(is)
      book = HSSFWorkbook.new(fs)
      @sheet = book.getSheetAt(0)
    rescue
      raise $!.message
    end
    is.close
  end

  def each
    i = @sheet.rowIterator
    while i.hasNext
      yield i.next
    end
  end

  def district_population
    DistrictPopulation.new(self)
  end

  def graph
    graph = Graph.new(600, 300)
    graph.draw(district_population.to_a)
    graph.png
  end

  def workbook
    hw = HSSFWorkbook.new
    sheet = hw.createSheet
    hw.setSheetName(0, _('Population Tokyo'), HSSFWorkbook.ENCODING_UTF_16)
    sheet.setColumnWidth(0, 256 * 12)
    row = sheet.createRow(0)
    [_('District'), _('Total'), _('Men'), _('Women')].each_with_index do |s, i|
      set_string(row.createCell(i), s)
    end
    district_population.each_with_index do |r, i|
      row = sheet.createRow(i + 1)
      set_string(row.createCell(0), r.name)
      [r.total, r.men, r.women].each_with_index do |s, i|
        row.createCell(i + 1).setCellValue(s)
      end
    end
    bout = ByteArrayOutputStream.new
    hw.write(bout)
    bout.toByteArray
  end
  
  private

  def set_string(c, s)
    c.setEncoding(HSSFCell.ENCODING_UTF_16)
    c.setCellValue(s)
  end
end
